@@ -14,8 +14,8 @@ Example::
     >>> from trigger.netdevices import NetDevices
     >>> nd = NetDevices()
     >>> dev = nd['test1-abc.net.aol.com']
-    >>> dev.manufacturer, dev.make
-    ('JUNIPER', 'MX960-BASE-AC')
+    >>> dev.vendor, dev.make
+    (<Vendor: Juniper>, 'MX960-BASE-AC')
     >>> dev.bounce.next_ok('green')
     datetime.datetime(2010, 4, 9, 9, 0, tzinfo=<UTC>)
 
@@ -24,32 +24,38 @@ Example::
 __author__ = 'Jathan McCollum, Eileen Tschetter, Mark Thomas, Michael Shields'
 __maintainer__ = 'Jathan McCollum'
 __email__ = 'jathan.mccollum@teamaol.com'
-__copyright__ = 'Copyright 2006-2011, AOL Inc.'
+__copyright__ = 'Copyright 2006-2012, AOL Inc.'
+__version__ = '1.4.0'
 
 # Imports (duh?)
 import itertools
 import os
-import sqlite3 as sqlite
 import sys
 import time
+from twisted.python import log
 from UserDict import DictMixin
-from xml.etree.cElementTree import ElementTree, parse
 from trigger.conf import settings
 from trigger.changemgmt import site_bounce, BounceStatus
 from trigger.acl.db import AclsDB
+from trigger.utils import network
+from trigger import exceptions
+from trigger import rancid
 
+# Parser imports
 try:
     import simplejson as json # Prefer simplejson because of SPEED!
 except ImportError:
     import json
+import sqlite3
+from xml.etree.cElementTree import parse
 
 
 # Constants
-SUPPORTED_FORMATS = ('xml', 'json', 'sqlite')
+SUPPORTED_FORMATS = ('json', 'rancid', 'sqlite', 'xml')
 
 
 # Exports
-__all__ = ['device_match', 'NetDevice', 'NetDevices']
+__all__ = ['device_match', 'NetDevice', 'NetDevices', 'Vendor']
 
 
 # Functions
@@ -57,7 +63,8 @@ def _parse_json(data_source):
     """
     Parse 'netdevices.json' and return list of JSON objects.
 
-    :param data_source: Absolute path to data file
+    :param data_source:
+        Absolute path to data file
     """
     with open(data_source, 'r') as contents:
          # TODO (jathan): Can we somehow return an generator like the other
@@ -71,7 +78,8 @@ def _parse_xml(data_source):
     Parse 'netdevices.xml' and return a list of node 2-tuples (key, value).
     These are as good as a dict without the extra dict() call.
 
-    :param data_source: Absolute path to data file
+    :param data_source:
+        Absolute path to data file
     """
     # Parsing the complete file into a tree once and extracting outthe device
     # nodes is faster than using iterparse(). Curses!!
@@ -86,9 +94,10 @@ def _parse_sqlite(data_source):
     """
     Parse 'netdevices.sql' and return a list of stuff.
 
-    :param data_source: Absolute path to data file
+    :param data_source:
+        Absolute path to data file
     """
-    connection = sqlite.connect(data_source)
+    connection = sqlite3.connect(data_source)
     cursor = connection.cursor()
 
     # Get the column names. This is a simple list strings.
@@ -107,8 +116,25 @@ def _parse_sqlite(data_source):
 
     return data
 
+def _parse_rancid(rancid_root,
+                  recurse_subdirs=settings.RANCID_RECURSE_SUBDIRS):
+    """
+    Parse RANCID's ``router.db`` and return a generator of node 2-tuples (key,
+    value).
+
+    :param rancid_root:
+        Absolute path to the RANCID directory
+
+    :param recurse_subdirs:
+        Whether to treat RANCID as single or multiple instance
+    """
+    data = rancid.parse_rancid_data(rancid_root,
+                                    recurse_subdirs=recurse_subdirs)
+
+    return data
+
 def _munge_source_data(data_source=settings.NETDEVICES_FILE,
-                       format=settings.NETDEVICES_FORMAT):
+                       data_format=settings.NETDEVICES_FORMAT):
     """
     Read the source data in the specified format, parse it, and return a
     dictionary of objects.
@@ -116,19 +142,21 @@ def _munge_source_data(data_source=settings.NETDEVICES_FILE,
     :param data_source: Absolute path to source data file
     :param format: One of 'xml', 'json', or 'sqlite'
     """
-    assert format in SUPPORTED_FORMATS
+    assert data_format in SUPPORTED_FORMATS
 
     parsers = {
         'xml': _parse_xml,
         'json': _parse_json,
         'sqlite': _parse_sqlite,
+        'rancid': _parse_rancid,
     }
-    parser = parsers[format]
+    parser = parsers[data_format]
     data = parser(data_source)
 
     return data
 
-def _populate(netdevices, data_source, data_format, production_only):
+def _populate(netdevices, data_source, data_format, production_only,
+              with_acls):
     """
     Populates the NetDevices with NetDevice objects.
 
@@ -136,36 +164,27 @@ def _populate(netdevices, data_source, data_format, production_only):
     objects.
     """
     #start = time.time()
-    aclsdb = AclsDB()
+    device_data = _munge_source_data(data_source=data_source,
+                                     data_format=data_format)
 
-    device_data = _munge_source_data(data_source=data_source, format=data_format)
+    # Populate AclsDB if `with_acls` is set
+    aclsdb = AclsDB() if with_acls else None
 
+    # Populate `netdevices` dictionary with `NetDevice` objects!
     for obj in device_data:
-        dev = NetDevice(data=obj)
+        dev = NetDevice(data=obj, with_acls=aclsdb)
 
         # Only return devices with adminStatus of 'PRODUCTION' unless
-        # production_only is True
+        # `production_only` is True
         if dev.adminStatus != 'PRODUCTION' and production_only:
+            #log.msg('DEVICE NOT PRODUCTION')
             continue
 
         # These checks should be done on generation of netdevices.xml.
-        ## skip empty nodenames
+        # Skip empty nodenames
         if dev.nodeName is None:
             continue
 
-        ## lowercase hostnames
-        dev.nodeName = dev.nodeName.lower()
-
-        ## cleanup whitespace from owning team
-        dev.owningTeam = dev.owningTeam.strip()
-
-        # Populate the ACLs for each device.
-        dev.explicit_acls = dev.implicit_acls = dev.acls = dev.bulk_acls = set()
-        acls_dict = aclsdb.get_acl_dict(dev)
-        dev.explicit_acls = acls_dict['explicit']
-        dev.implicit_acls = acls_dict['implicit']
-        dev.acls = acls_dict['all']
-
         # Add to dict
         netdevices[dev.nodeName] = dev
 
@@ -234,7 +253,7 @@ class NetDevice(object):
     Users usually won't create `NetDevice` objects directly! Rely instead upon
     `NetDevices` to do this for you.
     """
-    def __init__(self, data=None):
+    def __init__(self, data=None, with_acls=None):
         # Here comes all of the bare minimum set of attributes a NetDevice
         # object needs for basic functionality within the existing suite.
 
@@ -245,6 +264,7 @@ class NetDevice(object):
         self.deviceType = None
         self.make = None
         self.manufacturer = None
+        self.vendor = None
         self.model = None
         self.serialNumber = None
 
@@ -266,20 +286,89 @@ class NetDevice(object):
         self.room = None
         self.coordinate = None
 
-        # ACLs
+        # If `data` has been passed, use it to update our attributes
+        if data is not None:
+            self._populate_data(data)
+
+        # ACLs (defaults to empty sets)
         self.explicit_acls = self.implicit_acls = self.acls = self.bulk_acls = set()
+        if with_acls is not None:
+            self._populate_acls(aclsdb=with_acls)
 
-        # And if data has been passed, well... replace everything that was in
-        # it.
-        if data is None:
-            pass
-        else:
-            self.__dict__.update(data) # Better hope this is a dict!
+        # Cleanup the attributes (strip whitespace, lowercase values, etc.)
+        self._cleanup_attributes()
 
-        # And lowercase the nodeName for completeness.
+        # Map the manufacturer name to a Vendor object that has extra sauce
+        if self.manufacturer is not None:
+            self.vendor = vendor_factory(self.manufacturer)
+
+        # Use the vendor to populate the deviceType if it's not set already
+        if self.deviceType is None:
+            self._populate_deviceType()
+
+        # Bind the correct execute/connect methods based on deviceType
+        self._bind_dynamic_methods()
+
+    def _populate_data(self, data):
+        """
+        Populate the custom attribute data
+
+        :param data:
+            An iterable of key/value pairs
+        """
+        self.__dict__.update(data) # Better hope this is a dict!
+
+    def _cleanup_attributes(self):
+        """Perform various cleanup actions. Abstracted for customization."""
+        # Lowercase the nodeName for completeness.
         if self.nodeName is not None:
             self.nodeName = self.nodeName.lower()
 
+        # Cleanup whitespace from owning team
+        if self.owningTeam is not None:
+            self.owningTeam = self.owningTeam.strip()
+
+        # Map deviceStatus to adminStatus when data source is RANCID
+        if hasattr(self, 'deviceStatus'):
+            STATUS_MAP = {
+                'up': 'PRODUCTION',
+                'down': 'NON-PRODUCTION',
+            }
+            self.adminStatus = STATUS_MAP.get(self.deviceStatus, STATUS_MAP['up'])
+
+    def _populate_deviceType(self):
+        """Try to make a guess what the device type is"""
+        self.deviceType = settings.DEFAULT_TYPES.get(self.vendor.name,
+                                                     settings.FALLBACK_TYPE)
+
+    def _bind_dynamic_methods(self):
+        """
+        Bind dynamic methods to the instance. Currently does these:
+
+            + Dynamically bind ~trigger.twister.excute` to .execute()
+            + Dynamically bind ~trigger.twister.connect` to .connect()
+
+        Note that these both rely on the value of the ``vendor`` attribute.
+        """
+        from trigger import twister
+        self.execute = twister.execute.__get__(self, self.__class__)
+        self.connect = twister.connect.__get__(self, self.__class__)
+
+    def _populate_acls(self, aclsdb=None):
+        """
+        Populate the associated ACLs for this device.
+
+        :param aclsdb:
+            An `~trigger.acl.db.AclsDB` instance
+        """
+        if not aclsdb:
+            return None
+
+        acls_dict = aclsdb.get_acl_dict(self)
+        self.explicit_acls = acls_dict['explicit']
+        self.implicit_acls = acls_dict['implicit']
+        self.acls = acls_dict['all']
+
     def __str__(self):
         return self.nodeName
 
@@ -331,7 +420,56 @@ class NetDevice(object):
 
     def is_netscaler(self):
         """Am I a NetScaler?"""
-        return all([self.deviceType=='SWITCH', self.manufacturer=='CITRIX'])
+        return all([self.deviceType=='SWITCH', self.vendor=='citrix'])
+
+    def is_netscreen(self):
+        """Am I a NetScreen running ScreenOS?"""
+        return all([self.is_firewall(),
+                    self.vendor in ('juniper', 'netscreen')])
+
+    def is_ioslike(self):
+        """
+        Am I an IOS-like device (as determined by
+        ``settings.IOSLIKE_VENDORS``)?
+        """
+        return self.vendor in settings.IOSLIKE_VENDORS
+
+    def _ssh_enabled(self, disabled_mapping):
+        """Check whether vendor/type is enabled against the given mapping."""
+        disabled_types = disabled_mapping.get(self.vendor.name, [])
+        return self.deviceType not in disabled_types
+
+    def has_ssh(self):
+        """Am I even listening on SSH?"""
+        return network.test_tcp_port(self.nodeName, port=22)
+
+    def _can_ssh(self, method):
+        """
+        Am I enabled to use SSH for the given method in Trigger settings, and
+        if so do I even have SSH?
+
+        :param method: One of ('pty', 'async')
+        """
+        METHOD_MAP = {
+            'pty': settings.SSH_PTY_DISABLED,
+            'async': settings.SSH_ASYNC_DISABLED,
+        }
+        assert method in METHOD_MAP
+        method_enabled = self._ssh_enabled(METHOD_MAP[method])
+
+        return method_enabled and self.has_ssh()
+
+    def can_ssh_async(self):
+        """Am I enabled to use SSH async?"""
+        return self._can_ssh('async')
+
+    def can_ssh_pty(self):
+        """Am I enabled to use SSH pty?"""
+        return self._can_ssh('pty')
+
+    def is_reachable(self):
+        """Do I respond to a ping?"""
+        return network.ping(self.nodeName)
 
     def dump(self):
         """Prints details for a device."""
@@ -342,7 +480,8 @@ class NetDevice(object):
         print '\tOwning Team:      ', dev.owningTeam
         print '\tOnCall Team:      ', dev.onCallName
         print
-        print '\tManufacturer:     ', dev.manufacturer
+        print '\tVendor:           ', '%s (%s)' % (dev.vendor.title, dev.manufacturer)
+        #print '\tManufacturer:     ', dev.manufacturer
         print '\tMake:             ', dev.make
         print '\tModel:            ', dev.model
         print '\tType:             ', dev.deviceType
@@ -359,17 +498,101 @@ class NetDevice(object):
         print '\tLast Updated:     ', dev.lastUpdate
         print
 
+
+class Vendor(object):
+    """
+    Map a manufacturer name to Trigger's canonical name.
+
+    Given a manufacturer name like 'CISCO SYSTEMS', this will attempt to map it
+    to the canonical vendor name specified in ``settings.VENDOR_MAP``. If this
+    can't be done, attempt to split the name up ('CISCO, 'SYSTEMS') and see if
+    any of the words map. An exception is raised as a last resort.
+
+    This exposes a normalized name that can be used in the event of a
+    multi-word canonical name.
+    """
+    def __init__(self, manufacturer=None):
+        """
+        :param manufacturer:
+            The literal or "internal" name for a vendor that is to be mapped to
+            its canonical name.
+        """
+        if manufacturer is None:
+            raise SyntaxError('You must specify a `manufacturer` name')
+
+        self.manufacturer = manufacturer
+        self.name = self.determine_vendor(manufacturer)
+        self.title = self.name.title()
+
+    def determine_vendor(self, manufacturer):
+        """Try to turn the provided vendor name into the cname."""
+        vendor = settings.VENDOR_MAP.get(manufacturer)
+        if vendor is None:
+            mparts = [w for w in manufacturer.lower().split()]
+            for word in mparts:
+                if word in settings.SUPPORTED_VENDORS:
+                    vendor = word
+                    break
+                else:
+                    # Safe fallback to first word
+                    vendor = mparts[0]
+
+            # This breaks compatibility with officially unsupported devices
+            #else:
+            #    raise exceptions.UnsupportedVendor('No mapping found for %r in `settings.VENDOR_MAP`' % manufacturer)
+
+        return vendor
+
+    @property
+    def normalized(self):
+        """Return the normalized name for the vendor."""
+        return self.name.replace(' ', '_').lower()
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return '<%s: %s>' % (self.__class__.__name__, self.title)
+
+    def __eq__(self, other):
+        return self.name.__eq__(Vendor(str(other)).name)
+
+    def __contains__(self, other):
+        return self.name.__contains__(Vendor(str(other)).name)
+
+    def __hash__(self):
+        return hash(self.name)
+
+    def lower(self):
+        return self.normalized
+
+_vendor_registry = {}
+def vendor_factory(vendor_name):
+    """
+    Given a full name of a vendor, retrieve or create the canonical
+    `~trigger.netdevices.Vendor` object.
+
+    Vendor instances are cached to improve startup speed.
+
+    :param vendor_name:
+        The vendor's full manufacturer name (e.g. 'CISCO SYSTEMS')
+    """
+    return _vendor_registry.setdefault(vendor_name, Vendor(vendor_name))
+
+
 class NetDevices(DictMixin):
     """
     Returns an immutable Singleton dictionary of
-    :class:`~trigger.netdevices.NetDevice` objects. By default
-    it will only return devices for which ``adminStatus=='PRODUCTION'``.
+    `~trigger.netdevices.NetDevice` objects.
 
-    There are hardly any use cases where ``NON-PRODUCTION`` devices are needed, and
-    it can cause real bugs of two sorts:
+    By default it will only return devices for which
+    ``adminStatus=='PRODUCTION'``.
+
+    There are hardly any use cases where ``NON-PRODUCTION`` devices are needed,
+    and it can cause real bugs of two sorts:
 
       1. trying to contact unreachable devices and reporting spurious failures,
-      2. hot spares with the same nodeName.
+      2. hot spares with the same ``nodeName``.
 
     You may override this by passing ``production_only=False``.
     """
@@ -396,14 +619,12 @@ class NetDevices(DictMixin):
             TypeError: unbound method match() must be called with _actual
             instance as first argument (got str instance instead)
         """
-        def __init__(self, production_only=True):
+        def __init__(self, production_only, with_acls):
             self._dict = {}
-            _populate(
-                self._dict,
-                settings.NETDEVICES_FILE,
-                settings.NETDEVICES_FORMAT,
-                production_only,
-            )
+            _populate(netdevices=self._dict,
+                      data_source=settings.NETDEVICES_FILE,
+                      data_format=settings.NETDEVICES_FORMAT,
+                      production_only=production_only, with_acls=with_acls)
 
         def __getitem__(self, key):
             return self._dict[key]
@@ -480,7 +701,7 @@ class NetDevices(DictMixin):
 
                 >>> nd = NetDevices()
                 >>> myargs = {'onCallName':'Data Center', 'model':'FCSLB'}
-                >>> mydevices = nd(**kwargs)
+                >>> mydevices = nd(**myargs)
 
             Example by keyword arguments::
 
@@ -520,25 +741,30 @@ class NetDevices(DictMixin):
             """
             Returns a list of NetDevice objects with deviceType matching type.
 
-            Known deviceTypes: ['FIREWALL', 'ROUTER', 'SWITCH', 'DWDM']
+            Known deviceTypes: ['FIREWALL', 'ROUTER', 'SWITCH']
             """
             return [x for x in self._dict.values() if x.deviceType == devtype]
 
         def list_switches(self):
-            """Returns a list of NetDevice objects with deviceType of SWITCH """
+            """Returns a list of NetDevice objects with deviceType of SWITCH"""
             return self.get_devices_by_type('SWITCH')
 
         def list_routers(self):
-            """ Returns a list of NetDevice objects with deviceType of ROUTER """
+            """Returns a list of NetDevice objects with deviceType of ROUTER"""
             return self.get_devices_by_type('ROUTER')
 
         def list_firewalls(self):
-            """ Returns a list of NetDevice objects with deviceType of FIREWALL """
+            """Returns a list of NetDevice objects with deviceType of FIREWALL"""
             return self.get_devices_by_type('FIREWALL')
 
-    def __init__(self, production_only=True):
+    def __init__(self, production_only=True, with_acls=True):
+        """
+        :param production_only:
+            Whether to require devices to have ``adminStatus=='PRODUCTION'``.
+        """
         if NetDevices._Singleton is None:
-            NetDevices._Singleton = NetDevices._actual(production_only=production_only)
+            NetDevices._Singleton = NetDevices._actual(production_only=production_only,
+                                                       with_acls=with_acls)
 
     def __getattr__(self, attr):
         return getattr(NetDevices._Singleton, attr)
